/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef _vdkodbcpriv_h
#define _vdkodbcpriv_h

#include <vdkodbc/vdkodbc.h>

/*!
\class VDKODBCPriv
\brief Contains data base objects privileges
*/
class VDKODBCPriv
{
	protected:
		//! Grantor name
		VDKString	Grantor;
		//! Grantee name
		VDKString	Grantee;
		//! Privilege
		VDKString	Privilege;
		//! Grantable
		VDKString	Grantable;
	public:
		/*!
		Constructor
		*/
		VDKODBCPriv() {};
		/*!
		Destructor
		*/
		~VDKODBCPriv() {};
		/*!
		Returns name of the user who granted the privilege,
		isNull() if not applicable to the data source.
		\warning May return isNull() string
		*/
		VDKString GetGrantor() {
			return Grantor;
		};
		/*!
		Returns name of the user to whom the privilege was granted.
		\warning May return isNull() string
		*/
		VDKString GetGrantee() {
			return Grantee;
		};
		/*!
		Returns privilege string
		When identifying a column privilege.
		May be one of the following (or others supported by the data source when implementation-defined): 
		- SELECT: The grantee is permitted to retrieve data for the column.
		- INSERT: The grantee is permitted to provide data for the column in new rows
			that are inserted into the associated table.
		- UPDATE: The grantee is permitted to update data in the column.
		- REFERENCES: The grantee is permitted to refer to the column within a constraint
			(for example, a unique, referential, or table check constraint). \n
		When identifying a table privilege. May be one of the following or a data source-specific privilege. 
		- SELECT: The grantee is permitted to retrieve data for one or more columns of the table.
		- INSERT: The grantee is permitted to insert new rows containing data for one or more columns into the table.
		- UPDATE: The grantee is permitted to update the data in one or more columns of the table.
		- DELETE: The grantee is permitted to delete rows of data from the table.
		- REFERENCES: The grantee is permitted to refer to one or more columns of the table within a constraint
			(for example, a unique, referential, or table check constraint).
		The scope of action permitted the grantee by a given table privilege is data source-dependent.
		For example, the UPDATE privilege might permit the grantee to update all columns in a table
		on one data source and only those columns for which the grantor has
		the UPDATE privilege on another data source.
		\warning May return isNull() string
		*/
		VDKString GetPrivilege() {
			return Privilege;
		};
		/*!
		Returns grantable string
		Indicates whether the grantee is permitted to grant the privilege
		to other users; "YES", "NO", or isNull() if unknown or
		not applicable to the data source. 
		A privilege is either grantable or not grantable but not both.
		\warning May return isNull() string
		*/
		VDKString GetGrantable() {
			return Grantable;
		};
		/*!
		Set attribute value
		\param index Attribute type to set
		\param value New value
		Possible values for index :
		- VDKODBC_PRIV_GRANTOR		grantor name
		- VDKODBC_PRIV_GRANTEE		grantee name
		- VDKODBC_PRIV_PRIV			privilege
		- VDKODBC_PRIV_GRANTABLE	grantable
		For internal use only.
		*/
		void SetValue(int index, const char *value);
};

#endif
