/*
   GUI test program for VDKODBC
*/

#include "testvdkodbc.h"

static char* defaultFont = "terminal 12";
//static char* defaultFontBold = "courier bold 12";
//static char* defaultFontItalic = "courier oblique 12";

/********************************************************
 VdkodbcSQL SQL DESKTOP FORM CLASS
********************************************************/

// signal map
DEFINE_SIGNAL_MAP(VdkodbcSQL, VDKForm)
ON_SIGNAL(mnuQuit,activate_signal,OnmnuQuitActivate),
ON_SIGNAL(mnuExecute,activate_signal,OnmnuExecuteActivate)
END_SIGNAL_MAP
/*
*/
VdkodbcSQL::VdkodbcSQL(VDKForm* owner, char* title,
					   VDKODBCConnection *connection) :
	VDKForm(owner, title)
{
	tokenlist = NULL;
	Connection = connection;
	Query = new VDKODBCQuery("query", Connection);
}

void
VdkodbcSQL::GUISetup(void)
{
	SetSize(540, 380);
	Menubar = new VDKMenubar(this);
	Add(Menubar,0,0,0,0);
	Menubar->BorderWidth(1);
	File = new VDKMenuItem(this, "File", NULL);
	File->Caption = "File";
	File->Checked = false;
	gtk_widget_set_name (GTK_WIDGET(File->WrappedWidget()), "File");
	Menubar->Add(File,l_justify,1,1,0);
	Menu0 = new VDKMenu(this);
	File->Add(Menu0);
	mnuQuit = new VDKMenuItem(this,"mnuQuit",NULL);
	mnuQuit->Caption = "_Quit";
	mnuQuit->Checked = false;
	gtk_widget_set_name (GTK_WIDGET(mnuQuit->WrappedWidget()), "mnuQuit");
	Menu0->Add(mnuQuit,l_justify,1,1,0);
	mnuExecute = new VDKMenuItem(this,"mnuExecute",NULL);
	mnuExecute->Caption = "_Execute";
	mnuExecute->Checked = false;
	gtk_widget_set_name (GTK_WIDGET(mnuExecute->WrappedWidget()), "mnuExecute");
	Menu0->Add(mnuExecute,l_justify,1,1,0);
	HPaned = new VDKPaned(this, v_box);
	Add(HPaned,0,1,1,0);
	Vbox1 = new VDKBox(this,v_box);
	HPaned->Add(Vbox1,c_justify,1,1,0);
	SQLEditor = new VDKEditor(this);
	SQLEditor->Font = new VDKFont(SQLEditor->Owner(),defaultFont);
	SQLEditor->NormalBackground = clIvory;
	SQLEditor->SetForeground(clBlue);
	SQLEditor->SetSize(-1, 150);
	SQLEditor->ShowLineNumbers = false;
/* Useless since it will not work for SQL - plm 2003/02/14
	tokenlist = SQLEditor->LoadTokens("tokens.db");
	SQLEditor->SetTokens(tokenlist);
	// makes some nice colors and fonts
	VDKColor* firebrick = new VDKColor(SQLEditor, "firebrick");
	VDKColor* navyblue = new VDKColor(SQLEditor, "navy blue");
	VDKFont* bold = new VDKFont(SQLEditor, defaultFontBold);
	VDKFont* italic = new VDKFont(SQLEditor, defaultFontItalic);
	VDKColor* forest_green = new VDKColor(SQLEditor, "forest green");
	VDKColor* blue = new VDKColor(SQLEditor, "blue");
	// install syntax and patterns table
	SQLEditor->InstallSyntaxTable (
							  firebrick, bold, // keywords 
							  firebrick, NULL, // gtk+ names
							  forest_green, NULL, // macros
							  forest_green, bold, // preprocessor directives
							  blue, NULL, // constants
							  navyblue,italic); // remarks
*/
	Vbox1->Add(SQLEditor);
	Vbox2 = new VDKBox(this,v_box);
	HPaned->Add(Vbox2,l_justify,1,1,0);
	Scrolled = new VDKScrolled(this);
	SQLAns = new VDKTreeView(this, NULL, GTK_SELECTION_SINGLE);
	gtk_widget_set_name (GTK_WIDGET(SQLAns->WrappedWidget()), "SQLAns");
	Scrolled->AddWithoutViewport(SQLAns, l_justify, false, false, 0);
	Vbox2->Add(Scrolled);
	statusbar = new VDKStatusbar(this);
	gtk_widget_set_name (GTK_WIDGET(statusbar->WrappedWidget()), "statusbar");
	Add(statusbar,r_justify,false,false,0);
}

bool
VdkodbcSQL::CanClose()
{
	if (tokenlist) delete tokenlist;
	return true;
}

//signal response method
bool
VdkodbcSQL::OnmnuQuitActivate(VDKObject*)
{
	Close();
	return true;
}

//signal response method
bool
VdkodbcSQL::OnmnuExecuteActivate(VDKObject*)
{
	ExecuteQuery();
	return true;
}

/*
*/
void
VdkodbcSQL::ExecuteQuery()
{
	if (!Query) return;
	// Should be interesting to select only
	// a part of editor buffer - plm 2003/02/15
	VDKString query = SQLEditor->GetChars(0, -1);
	if (query.isEmpty()) return;
	query = utf8_to_l(query.c_str());
	printf("Query=%s\n", query.c_str());
	if (!Query->SQL_OK(Query->ExecDirect(query))) {
		Application()->VDKMessageBox("VDKODBC",
			(char*)Query->GetLastError().c_str(),
			VDK_ICONERROR | VDK_OK, NULL, NULL, 0);
		return;
	}
	statusbar->Pop();
	Vbox2->RemoveObject(Scrolled);
	Scrolled = new VDKScrolled(this);
	SQLAns = new VDKTreeView(this, NULL, GTK_SELECTION_SINGLE);
	gtk_widget_set_name (GTK_WIDGET(SQLAns->WrappedWidget()), "SQLAns");
	Scrolled->AddWithoutViewport(SQLAns, l_justify, false, false, 0);
	Vbox2->Add(Scrolled);
	// Is it a select statement ?
	VDKString keyword = query;
	keyword.LTrim();
	keyword.GetPart(1, ' ');
	keyword.UpperCase();
	if (keyword != VDKString("SELECT")) {
		ShowResult(false);
		return;	// All is done
	}
	// It is a select statement => we have to fetch the data
	ShowData();
	Query->Close();
}

/*
*/
void
VdkodbcSQL::ShowResult(bool select)
{
	VDKString tmp;

	if (select) {
		int nbrows = Query->RowCount();
		if (nbrows > 1) tmp.Sprintf(" %d rows fetched", nbrows);
		else tmp.Sprintf(" %d row fetched", nbrows);
	}
	else tmp.Sprintf(" Query executed");
	statusbar->Pop();
	statusbar->Push((char*)tmp.c_str());
}

/*
*/
void
VdkodbcSQL::ShowData()
{
	VDKString tmp;

	int nbcols = Query->ColumnCount();
	GType* types = new GType[nbcols];
	for (int i = 0; i < nbcols; i++) types[i] = G_TYPE_STRING;
	VDKTreeViewModel* model = new VDKTreeViewModel(types, nbcols);
	SQLAns->Model = model;
	delete types;
	for (int j = 0; j < nbcols; j++) {
		tmp = Query->GetColumnName(j+1);
		VDKTreeViewColumn* column = new VDKTreeViewColumn(SQLAns, j, (char*)tmp.c_str());
		column->Foreground = VDKRgb(COLOR_LIST);
		VDKFont* font = new VDKFont(Owner(), FONT_RESULT_LIST);
		column->Font = font;
	}
	ShowRows(model);
	ShowResult();
}

/*
*/
void
VdkodbcSQL::ShowRows(VDKTreeViewModel *model)
{
	int colnum;
	GtkTreeIter iter;
	VDKString sbuffer;

/* This is one way which doesn't work if RowCount miss to send a right value
   This happens on MSDE for example, it seems to depend on ODBC driver
	for (int k = 1; k <= Query->RowCount(); k++) {
		if (!Query->SQL_OK(Query->Fetch())) continue;
		model->AppendBlank(&iter);
		for (int j = 0; j < Query->ColumnCount(); j++) {
			colnum = j+1;
			sbuffer = Query->GetFieldValue(colnum);
			switch(Query->GetColumnType(colnum)) {
			  case SQL_NUMERIC:
				// + 1 for the decimal point
				sbuffer.LPad(Query->GetColumnSize(colnum) + 
						Query->GetColumnDecDigits(colnum) + 1, ' ');
				break;
			  case SQL_DATE:
			  case SQL_TYPE_DATE:
				sbuffer.FormatDate('/', INT_DATE, EUR_DATE);
				break;
			}
			printf("col=%d, val=%s\n", colnum, sbuffer.c_str());
			model->SetCell(&iter, j, l_to_utf8(sbuffer.c_str()));
		}
	}
*/
	while (!Query->Eof()) {
		model->AppendBlank(&iter);
		for (int j = 0; j < Query->ColumnCount(); j++) {
			colnum = j+1;
			sbuffer = Query->GetFieldValue(colnum);
			switch(Query->GetColumnType(colnum)) {
			  case SQL_NUMERIC:
				// + 1 for the decimal point
				sbuffer.LPad(Query->GetColumnSize(colnum) + 
						Query->GetColumnDecDigits(colnum) + 1, ' ');
				break;
			  case SQL_DATE:
			  case SQL_TYPE_DATE:
				sbuffer.FormatDate('/', INT_DATE, EUR_DATE);
				break;
			}
			printf("col=%d, val=%s\n", colnum, sbuffer.c_str());
			model->SetCell(&iter, j, l_to_utf8(sbuffer.c_str()));
		}
	}
}

/********************************************************
 Testvdkodbc LOGIN FORM CLASS
********************************************************/

// signal map
DEFINE_SIGNAL_MAP(VdkodbcLogin, VDKForm)
  ON_SIGNAL(OK,clicked_signal,OKClick),
  ON_SIGNAL(Cancel,clicked_signal,CancelClick)
END_SIGNAL_MAP

/*
*/
VdkodbcLogin::VdkodbcLogin(VDKForm* owner, char* title,
						   VDKString *user, VDKString *pwd) :
						   VDKForm(owner, title)
{
	FinalUName = user;
	FinalPwd = pwd;
	VDKBox* mainbox = new VDKBox(this,v_box);
	VDKLabel *label = new VDKLabel(this,"User name :");
	mainbox->Add(label);
	UserName = new VDKEntry(this);
	mainbox->Add(UserName);
	label = new VDKLabel(this,"Password :");
	mainbox->Add(label);
	PassWord = new VDKEntry(this);
	PassWord->Hidden = true;
	mainbox->Add(PassWord);
	VDKBox* buttonbox = new VDKBox(this, h_box);
	OK = new VDKCustomButton(this,"OK");
	buttonbox->Add(OK);
	Cancel = new VDKCustomButton(this,"Cancel");
	buttonbox->Add(Cancel);
	mainbox->Add(buttonbox);
	Add(mainbox);
	SetSize(200, -1);
	if (!user->isEmpty()) {
		UserName->Text = (char*)user->c_str();
		PassWord->GrabFocus();
	}
}

/*
 */
bool
VdkodbcLogin::OKClick(VDKObject*)
{
	*FinalUName = UserName->Text;
	*FinalPwd = PassWord->Text;
	Close();
	return true;
}

/*
 */
bool
VdkodbcLogin::CancelClick(VDKObject*)
{
	*FinalUName = "stop";
	*FinalPwd = "";
	Close();
	return true;
}

/********************************************************
 Testvdkodbc MAIN FORM CLASS
********************************************************/

//define static display type and initial form position
GtkWindowType TestvdkodbcForm::DisplayType = GTK_WINDOW_TOPLEVEL;
GtkWindowPosition TestvdkodbcForm::InitialPosition = GTK_WIN_POS_CENTER_ALWAYS;
/*
defining signal and events
dynamics tables
*/
DEFINE_SIGNAL_LIST(TestvdkodbcForm,VDKForm);
DEFINE_EVENT_LIST(TestvdkodbcForm,VDKForm);
/*
defining signal static table
*/
DEFINE_SIGNAL_MAP(TestvdkodbcForm,VDKForm)
ON_SIGNAL(mnuQuit,activate_signal,OnmnuQuitActivate),
ON_SIGNAL(mnuSQL,activate_signal,OnmnuSQLActivate),
ON_SIGNAL(tvDatabases,select_row_signal,OntvDatabasesRowSelected),
ON_SIGNAL(tvDatabases,row_activated_signal,OntvDatabasesRowActivated),
ON_SIGNAL(tvDatabases,click_column_signal,OntvDatabasesColumnClicked)
END_SIGNAL_MAP

//signal response method
bool 
TestvdkodbcForm::OntvDatabasesRowActivated(VDKObject* sender) 
{
	VDKTreeView* tree = dynamic_cast<VDKTreeView*>(sender);
	if(tree->Selections().size() == 0) tree->GetSelections();
	if(tree->Selections().size() == 1) {
		GtkTreeIter iter = tree->Selections()[0];
		VDKTreeViewModel* model = tree->Model;
		VDKString objtype = model->GetCell(&iter, OBJTYPE_COL);
		VDKString name = model->GetCell(&iter, NAME_COL);
		if (objtype == VDKString(ITEM_SOURCE)) 
		  ManageConnection(model, name, &iter);
	}
	tree->Selections().flush();
	return true;
}
 
//signal response method
bool 
TestvdkodbcForm::OntvDatabasesColumnClicked(VDKObject* sender) 
{
/*	
	Application()->VDKMessageBox("VDKODBC",
		"OntvDatabasesColumnClicked",
		VDK_OK, NULL, NULL, 0);
*/
	return true;
}
 
//signal response method
bool 
TestvdkodbcForm::OntvDatabasesRowSelected(VDKObject* sender) 
{
	statusbar->Pop();
	FlushList();
	VDKTreeView* tree = dynamic_cast<VDKTreeView*>(sender);
	if(tree->Selections().size() == 0) tree->GetSelections();
	if(tree->Selections().size() == 1) {
		GtkTreeIter iter = tree->Selections()[0];
		VDKTreeViewModel* model = tree->Model;
		VDKString objtype = model->GetCell(&iter, OBJTYPE_COL);
		VDKString name = model->GetCell(&iter, NAME_COL);
		if (objtype == VDKString(ITEM_SUPPORT)) {
			if (name == VDKString("Tables")) {
				GetTables();
				tree->Expand(&iter);
			}
			goto finish;
		}
		if (objtype == VDKString(ITEM_TABLE)) {
			GetFields(name);
			goto finish;
		}
		if (objtype == VDKString(ITEM_SOURCE)) statusbar->Push(HLP_SOURCE);
	}
finish:
	tree->Selections().flush();
	return true;
}
 
//signal response method
bool
TestvdkodbcForm::OnmnuSQLActivate(VDKObject* sender)
{
	if (!connect->Connected()) {
		Application()->VDKMessageBox("VDKODBC",
					"No source connected",
					VDK_OK, NULL, NULL, 0);
		return true;
	}
	SqlDesk = new VdkodbcSQL(this, "SQL Desktop", connect);
	if (!SqlDesk) return true;
	SqlDesk->Setup();
	SqlDesk->Show();
	return true;
}
 
//signal response method
bool
TestvdkodbcForm::OnmnuQuitActivate(VDKObject*)
{
	Close();
	return true;
}

/*
  main form gui setup
*/
void
TestvdkodbcForm::GUISetup(void)
{
	SetSize(600,450);
	Title = "VDKODBC 1.0.x Test";
	Menubar = new VDKMenubar(this);
	Add(Menubar,0,0,0,0);
	Menubar->BorderWidth(1);
	File = new VDKMenuItem(this, "File", NULL);
	File->Caption = "File";
	File->Checked = false;
	gtk_widget_set_name (GTK_WIDGET(File->WrappedWidget()), "File");
	Menubar->Add(File,l_justify,1,1,0);
	Menu0 = new VDKMenu(this);
	File->Add(Menu0);
	mnuQuit = new VDKMenuItem(this,"mnuQuit",NULL);
	mnuQuit->Caption = "_Quit";
	mnuQuit->Checked = false;
	gtk_widget_set_name (GTK_WIDGET(mnuQuit->WrappedWidget()), "mnuQuit");
	Menu0->Add(mnuQuit,l_justify,1,1,0);
	mnuQuery = new VDKMenuItem(this,"mnuQuery",NULL);
	mnuQuery->Caption = "Query";
	mnuQuery->Checked = false;
	gtk_widget_set_name (GTK_WIDGET(mnuQuery->WrappedWidget()), "mnuQuery");
	Menubar->Add(mnuQuery,l_justify,1,1,0);
	Menu1 = new VDKMenu(this);
	mnuQuery->Add(Menu1);
	mnuSQL = new VDKMenuItem(this,"mnuSQL",NULL);
	mnuSQL->Caption = "SQL _Desktop";
	mnuSQL->Checked = false;
	gtk_widget_set_name (GTK_WIDGET(mnuSQL->WrappedWidget()), "mnuSQL");
	Menu1->Add(mnuSQL,l_justify,1,1,0);
	HPaned = new VDKPaned(this,h_box);
	Add(HPaned,0,1,1,0);
	Vbox1 = new VDKBox(this,v_box);
	HPaned->Add(Vbox1,c_justify,1,1,0);
	VDKScrolled *scrolled = new VDKScrolled(this);
	MakeTreeView(scrolled);
	Vbox1->Add(scrolled);
	Vbox2 = new VDKBox(this,v_box);
	HPaned->Add(Vbox2,l_justify,1,1,0);
	Scrolled = new VDKScrolled(this);
	clist = new VDKTreeView(this, NULL, GTK_SELECTION_SINGLE);
	gtk_widget_set_name (GTK_WIDGET(tvDatabases->WrappedWidget()), "clist");
	Scrolled->AddWithoutViewport(clist, l_justify, false, false, 0);
	Vbox2->Add(Scrolled);
	statusbar = new VDKStatusbar(this);
	gtk_widget_set_name (GTK_WIDGET(statusbar->WrappedWidget()), "statusbar");
	Add(statusbar,r_justify,false,false,0);
}

/*
*/
void
TestvdkodbcForm::ManageConnection(VDKTreeViewModel *model, VDKString source, GtkTreeIter *iter)
{
	if (connect->GetDataSourceName() == source) {
		// Already connected to this source
		// Nothing to do here
		return;
	}
	if (connect->Connected()) {
		printf("Disconnected from '%s'\n",
			   connect->GetDataSourceName().c_str());
		// Flush concerned source node child(s)
		FlushSourceNode(connect->GetDataSourceName());
		connect->Disconnect();
	}
  retry:
	passwd = "";
	// we have to ask for a user name and a password here
	if (!GetLogin(source, &user, &passwd)) return;
	// Cancellation
	if (user.isEmpty()) return;
	if (user == VDKString("stop")) {
		user = "";
		return;
	}
	// Connection
	if (!connect->SQL_OK(connect->Connect(source.c_str(), user, passwd))) {
		printf("Connection missed on '%s' because %s\n",
			   source.c_str(), connect->GetLastError().c_str());
		Application()->VDKMessageBox("VDKODBC",
								  (char*)connect->GetLastError().c_str(),
								  VDK_ICONERROR | VDK_OK, NULL, NULL, 0);
		goto retry;
	}
	else {
		// Add 'Tables' item to tree view
		GtkTreeIter iter_child;
		model->AppendBlank(&iter_child, iter);
		model->SetData(&iter_child, NAME_COL, "Tables", DBNAME_COL, source.c_str(),
					   OBJTYPE_COL, ITEM_SUPPORT, LOADED_COL, false, -1);
		/*
		   Waiting for implementation... - plm 2003/02/14
		*/
		tvDatabases->Expand(iter);
		printf("Connected to '%s'\n", source.c_str());
	}
}

/*
*/
bool
TestvdkodbcForm::GetLogin(VDKString source, VDKString *user, VDKString *passwd)
{
	VDKString title;
	title.Sprintf("Login in %s", source.c_str());
	VdkodbcLogin *login = new VdkodbcLogin(this, (char*)title.c_str(), user, passwd);
	if (!login) return false;
	login->Setup();
	login->ShowModal();
	return true;
}

/*
*/
void
TestvdkodbcForm::MakeTreeView(VDKScrolled *scrolled)
{
	tvDatabases = new VDKTreeView(this, NULL, GTK_SELECTION_SINGLE);
#ifdef HIDE_COLUMNS
	tvDatabases->SetSize(200, -1);
#else
	tvDatabases->SetSize(340, -1);
#endif
	gtk_widget_set_name (GTK_WIDGET(tvDatabases->WrappedWidget()), "tvDatabases");
	GType* types = new GType[MAX_COLS];
	types[NAME_COL] = G_TYPE_STRING;
	types[DBNAME_COL] = G_TYPE_STRING;
	types[OBJTYPE_COL] = G_TYPE_STRING;
	types[LOADED_COL] = G_TYPE_BOOLEAN;
	tvDatabases->Model = new VDKTreeViewModel(types, MAX_COLS);
	delete types;
	VDKTreeViewColumn* column = new VDKTreeViewColumn(tvDatabases, NAME_COL, "unixODBC sources");
	column->Foreground = VDKRgb(COLOR_TV);
	VDKFont* font = new VDKFont(Owner(), FONT_TV);
	column->Font = font;
	column = new VDKTreeViewColumn(tvDatabases, DBNAME_COL, "Database name");
	// database name column is invisible
	column = new VDKTreeViewColumn(tvDatabases, OBJTYPE_COL, "Object type");
	// loaded column is invisible
	column = new VDKTreeViewColumn(tvDatabases, LOADED_COL, "Loaded");
	// loaded column is invisible
	scrolled->AddWithoutViewport(tvDatabases, l_justify, false, false, 0);
#ifdef HIDE_COLUMNS
	gtk_tree_view_column_set_visible ((*tvDatabases->Columns())[LOADED_COL]->GtkColumn(), false);
	gtk_tree_view_column_set_visible ((*tvDatabases->Columns())[OBJTYPE_COL]->GtkColumn(), false);
	gtk_tree_view_column_set_visible ((*tvDatabases->Columns())[DBNAME_COL]->GtkColumn(), false);
#endif
}

/*
*/
bool
TestvdkodbcForm::GetODBCDataSources()
{
	VDKString sources, source, tmp;
	GtkTreeIter iter;
	int i = 1;

	if (!connect) {
		printf("Not connected\n");
		return false;
	}
	// Get ODBC data sources
	if (connect->SQL_OK(connect->GetDataSources(sources))) {
		printf("Sources : \n---------\n%s\n", sources.c_str());
		tmp = sources;
		source = tmp.GetPart(i, '\n');
		while (!source.isEmpty()) {
			source.GetPart(1, '\t');
			printf("Source : %s\n", source.c_str());
			// Load each source into tree view
			VDKTreeViewModel* model = tvDatabases->Model;
			model->AppendBlank(&iter);
			model->SetData(&iter, NAME_COL, source.c_str(), DBNAME_COL, source.c_str(), 
					OBJTYPE_COL, ITEM_SOURCE, LOADED_COL, true, -1);
			tmp = sources;
			source = tmp.GetPart(++i, '\n');
		}
	}
	else return false;
	return true;
}

/*
*/
void
TestvdkodbcForm::GetFields(VDKString tablename)
{
	VDKString tmp;
	GtkTreeIter iter;

	VDKString source = connect->GetDataSourceName();
	if (connect->SQL_OK(connect->LoadColumnsAttr(tablename))) {
		Vbox2->RemoveObject(Scrolled);
		Scrolled = new VDKScrolled(this);
		clist = new VDKTreeView(this, NULL, GTK_SELECTION_SINGLE);
		gtk_widget_set_name (GTK_WIDGET(tvDatabases->WrappedWidget()), "clist");
		GType* types = new GType[MAX_FIELD_COLS];
		types[POS_FIELD_NAME] = G_TYPE_STRING;
		types[POS_FIELD_TYPE] = G_TYPE_STRING;
		types[POS_FIELD_SIZE] = G_TYPE_STRING;
		types[POS_FIELD_DEC]  = G_TYPE_STRING;
		types[POS_FIELD_NULL] = G_TYPE_BOOLEAN;
		// Suppressed because unixODBC driver doesn't deal with it
		// neither for PostgreSQL nor for MySQL - plm 2003/02/20
		//types[POS_FIELD_DEFV] = G_TYPE_STRING;
		VDKTreeViewModel* model = new VDKTreeViewModel(types, MAX_FIELD_COLS);
		clist->Model = model;
		delete types;
		for (int i = 0; i < MAX_FIELD_COLS; i++) {
			switch(i) {
				case POS_FIELD_NAME: tmp = "Name"; break;
				case POS_FIELD_TYPE: tmp = "Type"; break;
				case POS_FIELD_SIZE: tmp = "Size"; break;
				case POS_FIELD_DEC:  tmp = "Decimal"; break;
				case POS_FIELD_NULL: tmp = "Allow null"; break;
				//case POS_FIELD_DEFV: tmp = "Default value"; break;
			}
			VDKTreeViewColumn* column = new VDKTreeViewColumn(clist, i, (char*)tmp.c_str());
			if (i == POS_FIELD_NAME) column->Foreground = VDKRgb(COLOR_LIST_HEADER);
			else column->Foreground = VDKRgb(COLOR_LIST);
			VDKFont* font = new VDKFont(Owner(), FONT_LIST);
			column->Font = font;
		}
		for (unsigned int k = 0; k < connect->GetNbColumns(tablename); k++)
			AddField(tablename, model, &iter, k);
		Scrolled->AddWithoutViewport(clist, l_justify, false, false, 0);
		Vbox2->Add(Scrolled);
		ListEmpty = false;
	}
	else printf("Error n %d : %s\n", connect->GetLastNumError(), connect->GetLastError().c_str());
}

/*
*/
void
TestvdkodbcForm::AddField(VDKString tablename, VDKTreeViewModel *model,
						  GtkTreeIter *iter, unsigned int pos)
{
	VDKString columnname, type, size, dec;//, def;
	bool nul;

	columnname = connect->GetColumnName(tablename, pos);
	type = connect->GetColumnTypeName(tablename, columnname);
	size.Sprintf("%d", connect->GetColumnSize(tablename, columnname));
	dec.Sprintf("%d", connect->GetColumnDecDigits(tablename, columnname));
	nul = (connect->GetColumnNullable(tablename, columnname) == 1);
	//def = connect->GetColumnDefaultValue(tablename, columnname);
	model->AppendBlank(iter);
	model->SetData(iter,
			POS_FIELD_NAME, columnname.c_str(),
			POS_FIELD_TYPE, type.c_str(), 
			POS_FIELD_SIZE, size.c_str(),
			POS_FIELD_DEC, dec.c_str(),
			POS_FIELD_NULL, nul, -1);
/*
	printf("\t\tColumn %d : %s, %s, size:%s, len:%d, dec:%s, prec:%d, null:%d, default:%s, ord:%d\n",
		   pos, columnname.c_str(), type.c_str(), size.c_str(),
		   connect->GetColumnBufLength(tablename, columnname),
		   dec.c_str(), connect->GetColumnRadix(tablename, columnname),
		   nul, def.c_str(),
		   connect->GetColumnOrdinalPos(tablename, columnname));
	model->SetData(iter,
				   POS_FIELD_NAME, columnname.c_str(),
				   POS_FIELD_TYPE, type.c_str(), 
				   POS_FIELD_SIZE, size.c_str(),
				   POS_FIELD_DEC, dec.c_str(),
				   POS_FIELD_NULL, nul,
				   POS_FIELD_DEFV, def.c_str(), -1);
*/
}

/*
*/
void
TestvdkodbcForm::GetTables()
{
	VDKString tablename;

	GtkTreeIter iter, iter_parent = tvDatabases->Selections()[0];
	VDKTreeViewModel* model = tvDatabases->Model;
	VDKString val = model->GetCell(&iter_parent, LOADED_COL);
	if (val == VDKString(STRUE)) return;	// Already loaded => nothing to do
	VDKString source = connect->GetDataSourceName();
	if (source != VDKString(model->GetCell(&iter_parent, DBNAME_COL))) return;
	if (!connect->SQL_OK(connect->LoadTablesAttr())) {
		printf("Loading tables on '%s' missed because %s\n",
			   source.c_str(), connect->GetLastError().c_str());
	}
	else {
		printf("Tables quantity : %d\n", connect->GetNbTables());
		for (unsigned int j = 0; j < connect->GetNbTables(); j++) {
			tablename = connect->GetTableName(j);
			printf("Table num %d, name %s\n", j, tablename.c_str());
			model->AppendBlank(&iter, &iter_parent);
			model->SetData(&iter, NAME_COL, tablename.c_str(),
						   DBNAME_COL, source.c_str(),
						   OBJTYPE_COL, ITEM_TABLE, LOADED_COL, false, -1);
		}
		model->SetData(&iter_parent, LOADED_COL, true, -1);
	}
	tvDatabases->Selections().flush();
}

/*
*/
void
TestvdkodbcForm::FlushSourceNode(VDKString sourcename)
{
	FlushList();
	VDKTreeViewModel* model = tvDatabases->Model;
	if (!model) return;
	VDKTreeViewIter *iter = NULL;
	VDKTreeViewModelIterator ti(model, iter);
	for(;ti;ti++) {
		if (sourcename == VDKString(model->GetCell(ti.current(), NAME_COL))) {
			VDKTreeViewModelIterator tc(model, ti.current());
			model->Remove(tc.current());
		}
	}
}

/*
*/
void
TestvdkodbcForm::FlushList()
{
	if (ListEmpty) return;
	Vbox2->RemoveObject(Scrolled);
	Scrolled = new VDKScrolled(this);
	clist = new VDKTreeView(this, NULL, GTK_SELECTION_SINGLE);
	gtk_widget_set_name (GTK_WIDGET(tvDatabases->WrappedWidget()), "clist");
	Scrolled->AddWithoutViewport(clist, l_justify, false, false, 0);
	Vbox2->Add(Scrolled);
	ListEmpty = true;
}

/*
  main form constructor
*/
TestvdkodbcForm::TestvdkodbcForm(VDKApplication* app, char* title):
  VDKForm(app,title)
{
	ListEmpty = true;
	SqlDesk = NULL;
}

/*

*/
TestvdkodbcForm::~TestvdkodbcForm()
{
}

/*
  main form setup
*/
void
TestvdkodbcForm::Setup(void)
{
	GUISetup();
	// Creating connection object
	// One connection objet => Only one source connected at a time
	connect = new VDKODBCConnection();
	// Loading declared unixODBC sources
	if (GetODBCDataSources()) return;
}

/*
*/
bool
TestvdkodbcForm::CanClose(void)
{
	bool result = Application()->VDKMessageBox(
					  "VDKODBC \"ODBC for the Visual Development Kit\"",
					  "Really do you want quit VDKODBC test program ?",
					  VDK_ICONQUESTION | VDK_YESNO, NULL, NULL, 5000) == VDK_IDYES;
	if (result) {
		VDKTreeViewModel* model = tvDatabases->Model;
		model->Clear();
		if (connect->Connected())
			printf("Disconnected from '%s'\n",
				   connect->GetDataSourceName().c_str());
		// Disconnection, if needed, is automatic.
		delete connect;
	}
	return result;
}

/********************************************************
 Testvdkodbc APPLICATION CLASS
********************************************************/

/*
  application constructor
*/
TestvdkodbcApp::TestvdkodbcApp(int* argc, char** argv):
  VDKApplication(argc,argv)
{
}

/*
  application destructor
*/
TestvdkodbcApp::~TestvdkodbcApp()
{ 
}

/*
  application setup
*/

void
TestvdkodbcApp::Setup(void)
{
	SetGarbageCollection(2000);
	MainForm = new TestvdkodbcForm(this, NULL);
	MainForm->Setup();
	MainForm->Visible = true;
}

/*
  VDKODBC test program
*/
int main (int argc, char *argv[])
{
	// makes application
	TestvdkodbcApp app(&argc, argv);
	// parse resource file
	app.SetResourceFile("testvdkodbcrc");
	// runs application
	app.Run();
	return 0;
}

